/***************************************************************************
                          kmd.cpp  -  description
                             -------------------
    begin                : Tue Jan 15 00:49:05 MST 2002
    copyright            : (C) 2002 by Simon R
    email                : mail@srobins.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qcstring.h>
#include <qbitarray.h>

#include <stdlib.h>

#include <kapp.h>
#include <kdebug.h>
#include <kstddirs.h>
#include <klocale.h>
#include <kurl.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include "kmd.h"

using namespace KIO;
extern "C"{
  int kdemain( int argc, char **argv ){
    KInstance instance( "kio_kmd" );

    if (argc != 4){
			exit(-1);
    }

    kio_kmdProtocol slave(argv[2], argv[3]);
    slave.dispatchLoop();

    return 0;
  }
}

kio_kmdProtocol::kio_kmdProtocol(const QCString &pool_socket, const QCString &app_socket)
  : QObject(), SlaveBase("kio_kmd", pool_socket, app_socket)
{
}
/* ---------------------------------------------------------------------------------- */


kio_kmdProtocol::~kio_kmdProtocol()
{
}


/* ---------------------------------------------------------------------------------- */
void kio_kmdProtocol::get(const KURL& url )
{
  mimeType("text/plain");
  QString name = url.prettyURL();
	if (name.isEmpty() || name == "/"){
    data(QCString("USAGE: kmd:/mycommand -flags arg1 ..."));

	  data(QCString(name));
		finished();
		return;  	
	}else{
		name=name.mid(5); //lop of the kmd:/
	}
#ifndef NOCONFIRM
  if (messageBox(
		WarningContinueCancel, "About to execute:\n" + name, i18n("Warning"),"Execute"
	) == KMessageBox::Cancel){
	  finished();
		return;
	}
#endif
	infoMessage(i18n("Executing %1 ...").arg( name ) );
	QString exec = KGlobal::dirs()->findExe("sh");
	if(exec.isEmpty()){
		data(QCString("Could not find sh command"));
  	data(QByteArray());
  	finished();	
		return;
	}
	KProcess *shell = new KProcess;
	
	*shell << exec << "-c" << name;
	connect(shell, SIGNAL(receivedStdout(KProcess *,char *,int)), this, SLOT(shellStdout(KProcess *,char *,int)));
  connect(shell, SIGNAL(receivedStderr(KProcess *,char *,int)), this, SLOT(shellStdout(KProcess *,char *,int)));
	// run shell command
  _shellStdout.truncate(0);
  shell->start(KProcess::Block, KProcess::AllOutput);
	data(_shellStdout);
  data(QByteArray());
	infoMessage(i18n("Done"));
  finished();
	delete shell;
	_shellStdout.truncate(0);
}


/* --------------------------------------------------------------------------- */
void kio_kmdProtocol::shellStdout(KProcess */*proc*/, char *buffer, int buflen)
{
  _shellStdout += QCString(buffer).left(buflen);
}
